<?php

namespace Drupal\synprices\Plugin\Action;

use Drupal\views_bulk_operations\Action\ViewsBulkOperationsActionBase;
use Drupal\views_bulk_operations\Action\ViewsBulkOperationsPreconfigurationInterface;
use Drupal\Core\Plugin\PluginFormInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\commerce_price\Price;

/**
 * Action description.
 *
 * @Action(
 *   id = "synprices_update_prices_action",
 *   label = @Translation("Update Prices"),
 *   type = "",
 * )
 */
class UpdatePricesAction extends ViewsBulkOperationsActionBase implements ViewsBulkOperationsPreconfigurationInterface {
  /**
   * {@inheritdoc}
   */
  public function execute($entity = NULL) {
    if (empty($this->configuration['sale']) && empty($this->configuration['fixed_price'])) {
      return;
    }
    $entity->set('field_oldprice', $entity->price->number);
    if (!empty($this->configuration['fixed_price'])) {
      $new_price = new Price($this->configuration['fixed_price'], $entity->price->currency_code);
    }
    else {
      $new_price_number = $entity->price->number - ($entity->price->number * $this->configuration['sale'] / 100);
      $new_price = new Price($new_price_number, $entity->price->currency_code);
    }
    $entity->set('price', $new_price);
    $entity->save();
    return $this->t('Prices updated');
  }

  /**
   * Configuration form builder.
   *
   * If this method has implementation, the action is
   * considered to be configurable.
   *
   * @param array $form
   *   Form array.
   * @param Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   *
   * @return array
   *   The configuration form.
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
    $form['sale'] = [
      '#title' => t('Sale percent (%)'),
      '#type' => 'textfield',
      '#default_value' => $form_state->getValue('sale'),
    ];
    $form['fixed_price'] = [
      '#title' => t('Fixed price'),
      '#type' => 'textfield',
      '#default_value' => $form_state->getValue('fixed_price'),
    ];
    return $form;
  }

  /**
   * Submit handler for the action configuration form.
   *
   * If not implemented, the cleaned form values will be
   * passed direclty to the action $configuration parameter.
   *
   * @param array $form
   *   Form array.
   * @param Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state) {
    $this->configuration['sale'] = $form_state->getValue('sale');
    $this->configuration['fixed_price'] = $form_state->getValue('fixed_price');
  }

  /**
   * Build preconfigure action form elements.
   *
   * @param array $form
   *   Element of the views API form where configuration resides.
   * @param array $values
   *   Current values of the plugin pre-configuration.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form state interface object.
   *
   * @return array
   *   The action configuration form element.
   */
  public function buildPreConfigurationForm(array $form, array $values, FormStateInterface $form_state) : array {
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function access($object, AccountInterface $account = NULL, $return_as_object = FALSE) {
    // If certain fields are updated, access should be checked against them as well.
    // @see Drupal\Core\Field\FieldUpdateActionBase::access().
    return $object->access('update', $account, $return_as_object);
  }

}
